var prefs = {
  //for messenger toolbar
  icon_mode: 1, //0: icons and text, 1: icons, 2: text, -1: set individual
  placement: [],
  apply_multi: false,
  //for message window toolbar
  icon_mode_mw: 1,
  placement_mw: [],
  //for quick filter toolbar in messenger window
  icon_mode_qf: 0,
  placement_qf: []
};

//normal: messenger, messageDisplay: messageWindow, normal-QF: quick filter bar in messenger
var currentWindowType = "normal";

browser.windows.onCreated.addListener(async (win) => {
  if (win.type == "normal") {
    //messenger
    await browser.msgHdrCustomizeApi.reorderButtons(win.id, prefs.placement, (prefs.icon_mode == -1));
    if (prefs.icon_mode != -1) await browser.msgHdrCustomizeApi.setIconMode(win.id, prefs.icon_mode);
    
    //quick filter
    await browser.msgHdrCustomizeApi.reorderButtonsQuickFilter(prefs.placement_qf, (prefs.icon_mode_qf == -1));
    if (prefs.icon_mode_qf != -1) await browser.msgHdrCustomizeApi.setIconModeQuickFilter(prefs.icon_mode_qf);
  } else if (win.type == "messageDisplay") {
    //message window
    await browser.msgHdrCustomizeApi.reorderButtons(win.id, prefs.placement_mw, (prefs.icon_mode_mw == -1));
    if (prefs.icon_mode_mw != -1) await browser.msgHdrCustomizeApi.setIconMode(win.id, prefs.icon_mode_mw);
  }
});

browser.storage.onChanged.addListener(async (changes, area) => {
  let msgrChanged = false;
  let msgwinChanged = false;
  let qfChanged = false;

  for (let item in changes) {
    prefs[item] = changes[item].newValue;
    msgrChanged = msgrChanged || (item.indexOf("_mw") == -1 && item.indexOf("_qf") == -1);
    msgwinChanged = msgwinChanged || (item.indexOf("_mw") != -1);
    qfChanged = qfChanged || (item.indexOf("_qf") != -1);
  }

  if (msgrChanged) {
    changeHeaderForWindows("normal");
  }

  if (msgwinChanged) {
    changeHeaderForWindows("messageDisplay");
  }
  
  if (qfChanged) {
    changeHeaderForWindows("normal-QF");
  }
});

browser.runtime.onMessage.addListener(async (message) => {
  switch (message.message) {
    case "GET_BUTTONS":
      let buttons = [];
      let wins = await browser.windows.getAll();
      for (let i = 0; i < wins.length; i++) {
        let win = wins[i];
        let winType = currentWindowType.split("-")[0];
        let subType = currentWindowType.split("-")[1];
        
        if (win.type == "normal" && subType == "QF") {
          buttons = await browser.msgHdrCustomizeApi.getButtonsQuickFilter();
          break;
        } else if (win.type == winType) {
          buttons = await browser.msgHdrCustomizeApi.getButtons(win.id);
          break;
        }
      }
      browser.runtime.sendMessage({
        message: "SEND_BUTTONS",
        buttons: buttons,
        type: currentWindowType
      });
      break;
    case "CLOSE_REQ":
      await browser.windows.remove(message.windowId);
      break;  
    default:
      break;
  }
});

async function changeHeaderForWindows(type) {
  let wins = await browser.windows.getAll();
  let subType = type.split("-")[1];
  for (let i = 0; i < wins.length; i++) {
    let win = wins[i];
    if (win.type == "normal" && subType == "QF") {
      await browser.msgHdrCustomizeApi.reorderButtonsQuickFilter(prefs["placement_qf"], (prefs["icon_mode_qf"] == -1));
      if (prefs["icon_mode_qf"] != -1) await browser.msgHdrCustomizeApi.setIconModeQuickFilter(prefs["icon_mode_qf"]);
    } else if (win.type == type) {
      let suffix = (win.type == "messageDisplay") ? "_mw" : "";
      await browser.msgHdrCustomizeApi.reorderButtons(win.id, prefs["placement" + suffix], (prefs["icon_mode" + suffix] == -1));
      if (prefs["icon_mode" + suffix] != -1) await browser.msgHdrCustomizeApi.setIconMode(win.id, prefs["icon_mode" + suffix]);
    }
  }
}

async function init() {
  for (let key in prefs) {
    let result = await browser.storage.local.get(key);
    let value = result[key];
    if (value !== undefined) {
      prefs[key] = value;
    }
  }
  
  migrate20xTo21x();

  await browser.msgHdrCustomizeApi.init();

  browser.msgHdrCustomizeApi.onMenuClick.addListener(async (id) => {
    if (id.indexOf("MSG") != -1) {
      currentWindowType = "normal";
    } else if (id.indexOf("MW") != -1) {
      currentWindowType = "messageDisplay";
    } else if (id.indexOf("QF") != -1) {
      currentWindowType = "normal-QF";
    }
    
    await browser.windows.create({
      height: 450,
      width: 385,
      url: "html/options_window.html",
      type: "popup"
    });
  });

  browser.msgHdrCustomizeApi.onLoadMultiselect.addListener(async (id) => {
    if (prefs.apply_multi) {
      await browser.msgHdrCustomizeApi.reorderButtonsMulti(prefs.placement, (prefs.icon_mode == -1));
      if (prefs.icon_mode != -1) await browser.msgHdrCustomizeApi.setIconModeMulti(prefs.icon_mode);
    }
  });

  //for just after install
  changeHeaderForWindows("normal");
  changeHeaderForWindows("messageDisplay");
  changeHeaderForWindows("normal-QF");
}

function migrate20xTo21x() {
  for (let i=0; i<prefs.placement.length; i++) {
    let obj = prefs.placement[i];
    if (obj.iconMode === undefined) {
      obj.iconMode = prefs.icon_mode;
    }
    
    if (obj.subLabels === undefined) {
      obj.subLabels = [];
    }

    if (obj.renamed === undefined) {
      obj.renamed = false;
    }
  }
  
  for (let i=0; i<prefs.placement_mw.length; i++) {
    let obj = prefs.placement_mw[i];
    if (obj.iconMode === undefined) {
      obj.iconMode = prefs.icon_mode_mw;
    }
    
    if (obj.subLabels === undefined) {
      obj.subLabels = [];
    }
    
    if (obj.renamed === undefined) {
      obj.renamed = false;
    }
  }
  
  for (let i=0; i<prefs.placement_qf.length; i++) {
    let obj = prefs.placement_qf[i];
    if (obj.iconMode === undefined) {
      obj.iconMode = prefs.icon_mode_qf;
    }
    
    if (obj.subLabels === undefined) {
      obj.subLabels = [];
    }
    
    if (obj.renamed === undefined) {
      obj.renamed = false;
    }
  }
}

init();
