var {
  ExtensionCommon
} = ChromeUtils.import("resource://gre/modules/ExtensionCommon.jsm");
var {
  Services
} = ChromeUtils.import("resource://gre/modules/Services.jsm");

var {
  ExtensionSupport
} = ChromeUtils.import("resource:///modules/ExtensionSupport.jsm");

var localeStrings = {
  mhcCustomizeToolbar: "",
  mhcForwardInline: "",
  mhcForwardAttach: "",
  mhcForwardTooltip: "",
  mhcForwardInlineTooltip: "",
  mhcForwardAttachTooltip: "",
  mhcForwardAs: "",
  mhcTag: "",
  mhcTagTooltip: "",
  mhcNewTag: "",
  mhcManageTags: "",
  mhcConversation: "",
  mhcConversationTooltip: ""
};

//Define API
var msgHdrCustomizeApi = class extends ExtensionCommon.ExtensionAPI {
  getAPI(context) {
    return {
      msgHdrCustomizeApi: {
        init: async function () {
          for (let key in localeStrings) {
            localeStrings[key] = context.extension.localeData.localizeMessage(key);
          }

          return;
        },

        //mode -1: set individually, 0: both, mode 1: icon only, mode 2: text only
        setIconMode: async function (windowId, mode) {
          let rw = null;
          if (windowId != -1) {
            let windowObject = context.extension.windowManager.get(windowId);
            rw = windowObject.window;
          } else {
            rw = Services.wm.getMostRecentWindow("mail:3pane");
          }

          if (rw) {
            let toolbar = rw.document.getElementById("header-view-toolbar");
            if (toolbar) setIconMode(toolbar, mode);
          }
        },

        setIconModeMulti: async function (mode) {
          let rw = Services.wm.getMostRecentWindow("mail:3pane");

          if (rw) {
            //multiselection
            let multimsg = rw.document.getElementById("multimessage");
            let toolbar = multimsg ? multimsg.contentDocument.getElementById("header-view-toolbar") : null;
            if (toolbar) setIconMode(toolbar, mode);
          }
        },

        setIconModeQuickFilter: async function (mode) {
          let rw = Services.wm.getMostRecentWindow("mail:3pane");

          if (rw) {
            //multiselection
            let toolbar = rw.document.getElementById("quick-filter-bar-collapsible-buttons");
            if (toolbar) setIconMode(toolbar, mode);
          }
        },

        getButtons: async function (windowId) {
          let rw = null;
          if (windowId != -1) {
            let windowObject = context.extension.windowManager.get(windowId);
            rw = windowObject.window;
          } else {
            rw = Services.wm.getMostRecentWindow("mail:3pane");
          }

          let ret = [];
          if (rw) {
            let toolbar = rw.document.getElementById("header-view-toolbar");
            ret = getButtonsInToolbar(rw, toolbar);
          }

          return ret;
        },

        getButtonsQuickFilter: async function () {
          let rw = Services.wm.getMostRecentWindow("mail:3pane");
          let ret = [];
          if (rw) {
            let toolbar = rw.document.getElementById("quick-filter-bar-collapsible-buttons");
            if (toolbar) {
              let separator = toolbar.firstChild;
              toolbar.removeChild(separator);
              ret = getButtonsInToolbar(rw, toolbar);
              toolbar.insertBefore(separator, toolbar.firstChild);
            }
          }

          return ret;
        },

        reorderButtons: async function (windowId, order, setIcon) {
          let rw = null;
          if (windowId != -1) {
            let windowObject = context.extension.windowManager.get(windowId);
            rw = windowObject.window;
          } else {
            rw = Services.wm.getMostRecentWindow("mail:3pane");
          }

          if (rw) {
            let toolbar = rw.document.getElementById("header-view-toolbar");
            if (!toolbar) return false; //rw is other than messenger and message window

            //add custom buttons
            let conv = rw.document.getElementById("mhcConv");
            if (!conv) {
              createConversationButton(rw, toolbar);
            }
            let fwd = rw.document.getElementById("mhcFwdAs");
            if (!fwd) {
              createForwardAsButton(rw, toolbar);
            }
            let tag = rw.document.getElementById("mhcTag");
            if (!tag) {
              createTagButton(rw, toolbar)
            }

            if (order.length == 0) return false;

            reorderButtonsMain(rw, order, toolbar, setIcon, true);

            rw.UpdateJunkButton();
            rw.UpdateReplyButtons();
          }

          return true;
        },

        reorderButtonsMulti: async function (order, setIcon) {
          if (order.length == 0) return false;

          let rw = Services.wm.getMostRecentWindow("mail:3pane");

          if (rw) {
            let multimsg = rw.document.getElementById("multimessage");
            let toolbar = multimsg ? multimsg.contentDocument.getElementById("header-view-toolbar") : null;
            if (!toolbar) return false;

            reorderButtonsMain(rw, order, toolbar, setIcon, false);
          }

          return true;
        },

        reorderButtonsQuickFilter: async function (order, setIcon) {
          if (order.length == 0) return false;

          let rw = Services.wm.getMostRecentWindow("mail:3pane");

          if (rw) {
            let toolbar = rw.document.getElementById("quick-filter-bar-collapsible-buttons");
            if (!toolbar) return false;
            let separator = toolbar.firstChild;
            toolbar.removeChild(separator);
            reorderButtonsMain(rw, order, toolbar, setIcon, true);
            toolbar.insertBefore(separator, toolbar.firstChild);
          }
          return true;
        },

        onMenuClick: new ExtensionCommon.EventManager({
          context,
          name: "msgHdrCustomizeApi.onMenuClick",
          register(fire) {
            function callback(event, id) {
              return fire.async(id);
            }

            msgHdrCustomizeWindowListener.add(callback);
            return function () {
              msgHdrCustomizeWindowListener.remove(callback);
            };
          }
        }).api(),

        onLoadMultiselect: new ExtensionCommon.EventManager({
          context,
          name: "msgHdrCustomizeApi.onLoadMultiselect",
          register(fire) {
            function callback(event, id) {
              return fire.async(id);
            }

            msgHdrCustomizeMultiselectListener.add(callback);
            return function () {
              msgHdrCustomizeMultiselectListener.remove(callback);
            };
          }
        }).api()
      }
    };
  }
};

function setIconMode(obj, mode) {
  let labels = obj.querySelectorAll(".toolbarbutton-text");
  let images = obj.querySelectorAll(".toolbarbutton-icon");
  let badgestack = obj.querySelectorAll(".toolbarbutton-badge-stack");

  for (let i = 0; i < labels.length; i++) {
    labels[i].collapsed = (mode == 1);
  }
  for (let i = 0; i < images.length; i++) {
    images[i].collapsed = (mode == 2);
  }
  for (let i = 0; i < badgestack.length; i++) {
    badgestack[i].collapsed = (mode == 2);
  }
}

function getButtonsInToolbar(rw, toolbar) {
  let ret = [];
  let buttons = toolbar ? toolbar.childNodes : [];
  for (let i = 0; i < buttons.length; i++) {
    let button = buttons[i];

    let compStyles = null;
    if (button.localName == "toolbaritem") { //for Smart Reply icon
      //let childButton = button.querySelectorAll(".msgHeaderView-button")[0];
      let childButton = button.querySelectorAll(".toolbarbutton-1")[0];
      compStyles = rw.getComputedStyle(childButton);
    } else {
      compStyles = rw.getComputedStyle(button);
    }

    if (button.id) {
      let iconURL = compStyles.getPropertyValue("list-style-image");
      let id = button.id.replace(/^col-/, "");
      let labels = button.querySelectorAll(".toolbarbutton-text");
      let images = button.querySelectorAll(".toolbarbutton-icon");

      let labelCol = labels[0] ? labels[0].collapsed : false;
      let imageCol = images[0] ? images[0].collapsed : false;
      let iconMode = 0;
      if (labelCol) {
        iconMode = 1;
      } else if (imageCol) {
        iconMode = 2;
      }

      let subLabels = [];
      //Smart Reply button etc has several labels
      for (let i = 0; i < labels.length; i++) {
        subLabels.push(labels[i].value);
      }

      let renamed = button.getAttribute("renamed") == "true";

      ret.push({
        id: id,
        label: button.getAttribute("label"),
        collapsed: button.collapsed,
        iconURL: iconURL,
        iconMode: iconMode,
        subLabels: subLabels,
        renamed: renamed
      });
    }
  }

  return ret;
}

function reorderButtonsMain(rw, order, toolbar, setIcon, spacerEnabled) {
  let buttons = toolbar.childNodes;
  let buttonObjs = [];
  for (let i = buttons.length - 1; i >= 0; i--) {
    let button = buttons[i];
    let id = button.id;
    if (id) {
      id = id.replace(/^col-/, "");
      button.id = id;
    } else {
      id = "mhcTmpId" + i;
    }

    buttonObjs[id] = button;
    toolbar.removeChild(button);
  }

  let reordered = [];
  for (let i = 0; i < order.length; i++) {
    let id = order[i].id;
    let obj = buttonObjs[id];
    if (obj) {
      obj.collapsed = order[i].collapsed;
      if (obj.id) { //mhcTmpId button does not have id in actual
        obj.id = obj.collapsed ? "col-" + obj.id : obj.id;
      }

      setCustomLabel(obj, order[i]);

      //set icon display for each button
      if (setIcon && order[i].iconMode != undefined) {
        setIconMode(obj, order[i].iconMode);
      }

      toolbar.appendChild(obj);
    } else if (spacerEnabled && id.indexOf("mhcSPC") == 0) { //new Space or Separator
      obj = rw.document.createXULElement("toolbarspacer");
      obj.id = id;
      toolbar.appendChild(obj);
    } else if (spacerEnabled && id.indexOf("mhcSEP") == 0) {
      obj = rw.document.createXULElement("toolbarseparator");
      obj.id = id;
      toolbar.appendChild(obj);
    }
    reordered.push(id);
  }

  //add buttons not covered by order parameter or without ID
  for (let key in buttonObjs) {
    if (reordered.indexOf(key) == -1) {
      let obj = buttonObjs[key];
      if ((obj.id.indexOf("mhcSPC") != 0) && (obj.id.indexOf("mhcSEP") != 0)) {
        toolbar.appendChild(obj);
      }
    }
  }
}

function setCustomLabel(button, pref) {
  if (!pref.renamed) return;

  button.setAttribute("label", pref.label);
  button.setAttribute("renamed", true);

  let labels = button.querySelectorAll(".toolbarbutton-text");
  for (let i = 0; i < labels.length; i++) {
    if (pref.subLabels[i]) {
      labels[i].value = pref.subLabels[i];
    }
  }
}

function createConversationButton(rw, toolbar) {
  let button = rw.document.createXULElement("toolbarbutton");
  button.id = "mhcConv";
  button.classList.add("toolbarbutton-1");
  button.classList.add("msgHeaderView-button");
  button.setAttribute("label", localeStrings["mhcConversation"]);
  button.setAttribute("tooltiptext", localeStrings["mhcConversationTooltip"]);

  button.setAttribute("observes", "button_showconversation");
  button.style.listStyleImage = 'url("chrome://messenger/skin/icons/conversation.svg")';
  button.addEventListener("command", () => {
    rw.gConversationOpener.openConversationForMessages(rw.gFolderDisplay.selectedMessages);
    rw.RestoreFocusAfterHdrButton();
  });
  button.collapsed = true; //hidden by default
  toolbar.appendChild(button);
}

function createForwardAsButton(rw, toolbar) {
  let button = rw.document.createXULElement("toolbarbutton");
  button.id = "mhcFwdAs";
  button.setAttribute("is", "toolbarbutton-menu-button");
  button.setAttribute("type", "menu-button");
  button.setAttribute("label", localeStrings["mhcForwardAs"]); //not shown as button text. used for customize window.
  button.setAttribute("tooltiptext", localeStrings["mhcForwardTooltip"]);

  button.setAttribute("class", "toolbarbutton-1 msgHeaderView-button");
  button.style.listStyleImage = 'url("chrome://messenger/skin/icons/forward.svg")';
  button.setAttribute("observes", "button_forward");
  button.addEventListener("command", (event) => {
    rw.MsgForwardMessage(event);
    rw.RestoreFocusAfterHdrButton();
  });
  button.collapsed = true; //hidden by default

  let popup = rw.document.createXULElement("menupopup");
  popup.id = "mhcFwdAsPopup"
  let item = rw.document.createXULElement("menuitem");
  item.setAttribute("label", localeStrings["mhcForwardInline"]);
  item.setAttribute("tooltiptext", localeStrings["mhcForwardInlineTooltip"]);
  item.addEventListener("command", (event) => {
    rw.MsgForwardAsInline(event);
    event.stopPropagation();
    rw.RestoreFocusAfterHdrButton();
  });
  popup.appendChild(item);

  item = rw.document.createXULElement("menuitem");
  item.setAttribute("label", localeStrings["mhcForwardAttach"]);
  item.setAttribute("tooltiptext", localeStrings["mhcForwardAttachTooltip"]);
  item.addEventListener("command", (event) => {
    rw.MsgForwardAsAttachment(event);
    event.stopPropagation();
    rw.RestoreFocusAfterHdrButton();
  });
  popup.appendChild(item);
  button.appendChild(popup);

  //this button is needed to reproduce menu-button
  let childButton = rw.document.createXULElement("toolbarbutton");
  childButton.setAttribute("class", "box-inherit toolbarbutton-menubutton-button");
  childButton.flex = 1;
  childButton.setAttribute("allowevents", true);
  childButton.setAttribute("label", localeStrings["mhcForwardAs"]); //this label is shown as button text
  button.appendChild(childButton);

  let drop = rw.document.createXULElement("dropmarker");
  drop.setAttribute("type", "menu-button");
  drop.setAttribute("class", "toolbarbutton-menubutton-dropmarker");
  button.appendChild(drop);
  toolbar.appendChild(button);
}

function createTagButton(rw, toolbar) {
  let button = rw.document.createXULElement("toolbarbutton");
  button.id = "mhcTag";
  button.classList.add("toolbarbutton-1");
  button.classList.add("msgHeaderView-button");
  button.setAttribute("type", "menu");
  button.setAttribute("label", localeStrings["mhcTag"]);
  button.setAttribute("tooltiptext", localeStrings["mhcTagTooltip"]);
  button.style.listStyleImage = 'url("chrome://messenger/skin/icons/tag.svg")';
  button.setAttribute("wantdropmarker", "true");
  button.collapsed = true; //hidden by default

  let popup = rw.document.createXULElement("menupopup");
  popup.addEventListener("popupshowing", () => {
    rw.InitMessageTags(popup);
  });
  popup.id = "mhcTagPopup";

  let menuitem = rw.document.createXULElement("menuitem");
  menuitem.id = "mhcTagPopup-addNewTag";
  menuitem.setAttribute("label", localeStrings["mhcNewTag"]);
  menuitem.setAttribute("command", "cmd_addTag");
  menuitem.setAttribute("accesskey", "N");
  popup.appendChild(menuitem);

  menuitem = rw.document.createXULElement("menuitem");
  menuitem.id = "mhcTagPopup-manageTags";
  menuitem.setAttribute("label", localeStrings["mhcManageTags"]);
  menuitem.setAttribute("command", "cmd_manageTags");
  menuitem.setAttribute("accesskey", "M");
  popup.appendChild(menuitem);

  menuitem = rw.document.createXULElement("menuseparator");
  popup.appendChild(menuitem);

  menuitem = rw.document.createXULElement("menuitem");
  menuitem.id = "mhcTagPopup-tagRemoveAll";
  menuitem.setAttribute("command", "cmd_removeTags");
  //label and accesskey for remove all are set by InitMessageTags(popup)
  popup.appendChild(menuitem);

  menuitem = rw.document.createXULElement("menuseparator");
  popup.appendChild(menuitem);

  button.appendChild(popup);
  toolbar.appendChild(button);
}

var msgHdrCustomizeWindowListener = new class extends ExtensionCommon.EventEmitter {
  constructor() {
    super();
    this.callbackCount = 0;
  }

  handleEvent(event) {
    msgHdrCustomizeWindowListener.emit("menu-clicked", event.target.id);
  }

  add(callback) {
    this.on("menu-clicked", callback);
    this.callbackCount++;

    if (this.callbackCount == 1) {
      ExtensionSupport.registerWindowListener("msghdrcust_experimentListener", {
        chromeURLs: [
          "chrome://messenger/content/messenger.xhtml",
          "chrome://messenger/content/messageWindow.xhtml",
        ],
        onLoadWindow: function (window) {
          let isMessenger = window.location.href.indexOf("messenger.xhtml") != -1;
          let menuitemId = isMessenger ? "otherActionsCustomToolbarMSG" : "otherActionsCustomToolbarMW";
          let menuitem = window.document.getElementById(menuitemId);
          if (!menuitem) {
            let popup = window.document.getElementById("otherActionsPopup");
            let separator = window.document.createXULElement("menuseparator");
            separator.id = "mhcCustomizeSeparator";
            popup.appendChild(separator);
            menuitem = window.document.createXULElement("menuitem");
            menuitem.id = menuitemId;
            menuitem.label = localeStrings["mhcCustomizeToolbar"];
            menuitem.addEventListener("command", msgHdrCustomizeWindowListener.handleEvent);
            popup.appendChild(menuitem);
          }

          let popupset = window.document.getElementById("mainPopupSet");
          let menupopup = window.document.getElementById("mhcToolbarPopup");
          if (!menupopup) {
            let menupopup = window.document.createXULElement("menupopup");
            menupopup.id = "mhcToolbarPopup";

            let menuitem = window.document.createXULElement("menuitem");
            menuitem.id = isMessenger ? "mhcPopupCustomToolbarMSG" : "mhcPopupCustomToolbarMW";
            menuitem.label = localeStrings["mhcCustomizeToolbar"];
            menuitem.addEventListener("command", msgHdrCustomizeWindowListener.handleEvent);

            menupopup.appendChild(menuitem);
            popupset.appendChild(menupopup);

            let toolbox = window.document.getElementById("expandedHeadersTopBox");
            toolbox.setAttribute("context", "mhcToolbarPopup");
          }

          menupopup = window.document.getElementById("mhcToolbarPopupQF");
          if (!menupopup) {
            let qfToolbar = window.document.getElementById("quick-filter-bar");
            if (qfToolbar) {
              let menupopup = window.document.createXULElement("menupopup");
              menupopup.id = "mhcToolbarPopupQF";

              let menuitem = window.document.createXULElement("menuitem");
              menuitem.id = "mhcPopupCustomToolbarQF";
              menuitem.label = localeStrings["mhcCustomizeToolbar"];
              menuitem.addEventListener("command", msgHdrCustomizeWindowListener.handleEvent);

              menupopup.appendChild(menuitem);
              popupset.appendChild(menupopup);
              qfToolbar.setAttribute("context", "mhcToolbarPopupQF");
            }
          }
        }
      });
    }
  }

  remove(callback) {
    this.off("menu-clicked", callback);
    this.callbackCount--;

    if (this.callbackCount == 0) {
      for (let window of ExtensionSupport.openWindows) {
        if ([
          "chrome://messenger/content/messenger.xhtml",
          "chrome://messenger/content/messageWindow.xhtml",
        ].includes(window.location.href)) {
          let menuitem = window.document.getElementById("otherActionsCustomToolbar");
          if (menuitem) {
            let popup = window.document.getElementById("otherActionsPopup");
            menuitem.removeEventListener("command", this.handleEvent);
            popup.removeChild(menuitem);
            let separator = window.document.getElementById("mhcCustomizeSeparator");
            popup.removeChild(separator);
          }

          let menupopup = window.document.getElementById("mhcToolbarPopup");
          let popupset = window.document.getElementById("mainPopupSet");
          if (menupopup) {
            let toolbox = window.document.getElementById("expandedHeadersTopBox");
            if (toolbox) toolbox.removeAttribute("context")
            popupset.removeChild(menupopup);
          }

          menupopup = window.document.getElementById("mhcToolbarPopupQF");
          if (menupopup) {
            let qfToolbar = window.document.getElementById("quick-filter-bar");
            if (qfToolbar) qfToolbar.removeAttribute("context")
            popupset.removeChild(menupopup);
          }
        }
      }
      ExtensionSupport.unregisterWindowListener("msghdrcust_experimentListener");
    }
  }
};

var msgHdrCustomizeMultiselectListener = new class extends ExtensionCommon.EventEmitter {
  constructor() {
    super();
    this.callbackCount = 0;
  }

  handleEvent(event) {
    msgHdrCustomizeMultiselectListener.emit("multiselect", "multimessage");
  }

  add(callback) {
    this.on("multiselect", callback);
    this.callbackCount++;

    if (this.callbackCount == 1) {
      ExtensionSupport.registerWindowListener("msghdrcust_multiselectListener", {
        chromeURLs: [
          "chrome://messenger/content/messenger.xhtml"
        ],
        onLoadWindow: function (window) {
          let toolbox = window.document.getElementById("multimessage");
          toolbox.addEventListener("DOMContentLoaded", msgHdrCustomizeMultiselectListener.handleEvent);
        }
      });
    }
  }

  remove(callback) {
    this.off("multiselect", callback);
    this.callbackCount--;

    if (this.callbackCount == 0) {
      for (let window of ExtensionSupport.openWindows) {
        if ([
          "chrome://messenger/content/messenger.xhtml"
        ].includes(window.location.href)) {
          let toolbox = window.document.getElementById("multimessage");
          toolbox.removeEventListener("DOMContentLoaded", msgHdrCustomizeMultiselectListener.handleEvent);
        }
      }
      ExtensionSupport.unregisterWindowListener("msghdrcust_multiselectListener");
    }
  }
};
