var prefSuffix = "";
var iconModeDefault = {
  icon_mode: 1,
  icon_mode_mw: 1,
  icon_mode_qf: 0
};

async function init() {
  translate();
  await browser.runtime.sendMessage({
    message: "GET_BUTTONS"
  });

  let select = document.mhc_form.icon_mode;
  select.addEventListener("change", async () => {
    let mode = document.mhc_form.icon_mode.value;
    let obj = {};
    obj["icon_mode" + prefSuffix] = parseInt(mode);
    await browser.storage.local.set(obj);
    setButtonTextAndStyle();
    updateIconModeButtonsVisibility();
    updateIconModeButtons();
    updateButtonOrder();
  });

  let result = await browser.storage.local.get("apply_multi");
  let check = document.getElementById("apply_multi");
  check.checked = result.apply_multi ? true : false;
  check.addEventListener("change", async () => {
    let checked = document.getElementById("apply_multi").checked;
    await browser.storage.local.set({
      apply_multi: checked
    });
  });

  let buttonList = document.getElementById("placement");
  buttonList.addEventListener("change", async () => {
    updateShowHideButton();
    updateIconModeButtons();
    updateIconModeButtonsVisibility();
    enableButtons();
  });

  document.getElementById("buttonClose").addEventListener("click", async () => {
    let windowId = (await browser.windows.getCurrent()).id;
    await browser.runtime.sendMessage({
      message: "CLOSE_REQ",
      windowId: windowId
    });
  });
}

browser.runtime.onMessage.addListener(async (message) => {
  switch (message.message) {
    case "SEND_BUTTONS":
      if (message.type == "normal") {
        prefSuffix = "";
      } else if (message.type == "messageDisplay") {
        prefSuffix = "_mw";
      } else if (message.type == "normal-QF") {
        prefSuffix = "_qf";
      }

      await setIconMode();
      fillButtonList(message.buttons);
      setButtonTextAndStyle();
      if (prefSuffix == "") {
        document.getElementById("multiselArea").classList.remove("box-hidden");
        document.getElementById("multiselArea").classList.add("box-visible");
      }
      break;
    default:
      break;
  }
});

function setIconToOption(id, url) {
  let css = `
  #${id}:before{
    content: "";
    display: inline-block;
    width: 16px;
    height: 16px;
    background-image: ${url};
    background-repeat: no-repeat;
    background-size: contain;
    vertical-align: middle;
    padding-right: 5px;
  }`;

  document.styleSheets[0].insertRule(css);
}

async function setIconMode() {
  let result = await browser.storage.local.get("icon_mode" + prefSuffix);
  let select = document.mhc_form.icon_mode;
  select.value = result["icon_mode" + prefSuffix] === undefined ?
    iconModeDefault["icon_mode" + prefSuffix] :
    result["icon_mode" + prefSuffix];
  updateIconModeButtonsVisibility();
}

function fillButtonList(buttons) {
  let box = document.getElementById("placement");

  for (let i = box.childNodes.length - 1; i >= 0; i--) {
    box.removeChild(box.childNodes[i]);
  }

  let hiddenMsg = browser.i18n.getMessage("mhcHidden");
  for (let i = 0; i < buttons.length; i++) {
    let button = buttons[i];
    let option = document.createElement("option");
    if (button.id.indexOf("mhcSPC") == 0) {
      option.textContent = browser.i18n.getMessage("mhcSpace");
    } else if (button.id.indexOf("mhcSEP") == 0) {
      option.textContent = browser.i18n.getMessage("mhcSeparator");
    } else {
      setIconToOption(button.id, button.iconURL);
    }
    option.id = button.id;
    option.setAttribute("buttonLabel", button.label);
    option.setAttribute("buttonCollapsed", button.collapsed);
    option.setAttribute("buttonIconMode", button.iconMode);
    option.setAttribute("buttonId", button.id);
    option.setAttribute("buttonSubLabels", button.subLabels.join(", "));
    option.setAttribute("buttonRenamed", button.renamed);

    if (button.collapsed) {
      option.classList.add("option-hidden");
    }

    box.appendChild(option);
  }

  document.getElementById("buttonUp").addEventListener("click", () => {
    let moved = moveOptsInSelection(
      document.getElementById("placement"),
      true
    );
    if (moved) updateButtonOrder();
  });

  document.getElementById("buttonDown").addEventListener("click", () => {
    let moved = moveOptsInSelection(
      document.getElementById("placement"),
      false
    );
    if (moved) updateButtonOrder();
  });

  document.getElementById("buttonHide").addEventListener("click", () => {
    let changed = hideSelectedButtons(
      document.getElementById("placement")
    );
    if (changed) {
      updateShowHideButton();
      updateButtonOrder();
    }
  });

  document.getElementById("buttonIcon").addEventListener("click", () => {
    let changed = toggleIconForSelectedButtons(
      document.getElementById("placement")
    );
    if (changed) {
      updateButtonOrder();
    }
  });

  document.getElementById("buttonText").addEventListener("click", () => {
    let changed = toggleTextForSelectedButtons(
      document.getElementById("placement")
    );
    if (changed) {
      updateButtonOrder();
    }
  });

  document.getElementById("buttonSpc").addEventListener("click", async () => {
    let changed = addSpacer(
      document.getElementById("placement"),
      "mhcSPC"
    );
    if (changed) updateButtonOrder();
  });

  document.getElementById("buttonSep").addEventListener("click", async () => {
    let changed = addSpacer(
      document.getElementById("placement"),
      "mhcSEP"
    );
    if (changed) updateButtonOrder();
  });

  document.getElementById("buttonReset").addEventListener("click", async () => {
    await resetButtonOrder();
  });
  
  document.getElementById("buttonRename").addEventListener("click", async () => {
    let changed = renameButtons(document.getElementById("placement"));
    if (changed) updateButtonOrder();
  });

  enableButtons();
}

function setButtonTextAndStyle() {
  let hiddenMsg = browser.i18n.getMessage("mhcHidden");
  let mode = parseInt(document.mhc_form.icon_mode.value)

  let select = document.getElementById("placement");

  for (let i = 0; i < select.options.length; i++) {
    let option = select.options[i];
    if (option.id.indexOf("mhcSPC") == 0 || option.id.indexOf("mhcSEP") == 0) {
      continue;
    }

    let label = option.getAttribute("buttonLabel");
    if (mode >= 0) {
      option.setAttribute("buttonIconMode", mode);
    }
    if (option.getAttribute("buttonCollapsed") == "true") {
      option.textContent = label + " " + hiddenMsg;
      option.id = option.getAttribute("buttonId");
      option.id = "hide-" + option.getAttribute("buttonId");
    } else if (option.getAttribute("buttonIconMode") == "0") {
      option.textContent = label;
      option.id = option.getAttribute("buttonId");
    } else if (option.getAttribute("buttonIconMode") == "1") {
      option.textContent = "(" + label + ")";
      option.id = option.getAttribute("buttonId");
    } else if (option.getAttribute("buttonIconMode") == "2") {
      option.textContent = label;
      option.id = "noicon-" + option.getAttribute("buttonId");
    }
  }
}

function enableButtons() {
  let select = document.getElementById("placement");
  let opts = select.selectedOptions;

  if (opts.length > 0) {
    document.getElementById("buttonHide").disabled = false;
    document.getElementById("buttonSpc").disabled = (opts.length > 1);
    document.getElementById("buttonSep").disabled = (opts.length > 1);
    let firstIndex = getIndexOfOption(select, opts[0]);
    document.getElementById("buttonUp").disabled = (firstIndex == 0);
    let lastIndex = getIndexOfOption(select, opts[opts.length - 1]);
    document.getElementById("buttonDown").disabled = (lastIndex == select.options.length - 1);
    updateIconModeButtons();
  } else {
    document.getElementById("buttonUp").disabled = true;
    document.getElementById("buttonDown").disabled = true;
    document.getElementById("buttonHide").disabled = true;
    document.getElementById("buttonSpc").disabled = true;
    document.getElementById("buttonSep").disabled = true;
    document.getElementById("buttonIcon").disabled = true;
    document.getElementById("buttonText").disabled = true;
  }
}

function moveOptsInSelection(list, up) {
  if (list.selectedIndex == -1) return false;
  let orgOpts = list.selectedOptions;

  //copy to array. selectedOptions will change by the appendChild() and insertBefore()
  let opts = [];
  for (let i = 0; i < orgOpts.length; i++) {
    opts.push(orgOpts[i]);
  }

  let firstIndex = getIndexOfOption(list, opts[0]);
  let lastIndex = getIndexOfOption(list, opts[opts.length - 1]);
  if (up && firstIndex == 0) return false;
  if (!up && lastIndex == list.childNodes.length - 1) return false;

  if (up) {
    for (let i = 0; i < opts.length; i++) {
      let opt = opts[i];
      swapOptInSelection(list, up, opt);
    }
  } else {
    for (var i = opts.length - 1; i >= 0; i--) {
      let opt = opts[i];
      swapOptInSelection(list, up, opt);
    }
  }

  enableButtons();

  return true;
}

function swapOptInSelection(list, up, opt) {
  let refIndex = up ? getIndexOfOption(list, opt) - 1 : getIndexOfOption(list, opt) + 2;
  let refChild = list.childNodes[refIndex];
  if (refIndex >= list.childNodes.length) {
    list.appendChild(opt);
  } else {
    list.insertBefore(opt, refChild);
  }
}

function getIndexOfOption(list, opt) {
  let index = -1;
  for (let i = 0; i < list.length; i++) {
    if (list[i] === opt) {
      index = i;
      break;
    }
  }

  return index;
}

async function updateButtonOrder() {
  let box = document.getElementById("placement");
  let newOrder = [];
  for (let i = 0; i < box.options.length; i++) {
    let opt = box.options[i];
    let subLabels = [];
    if (opt.getAttribute("buttonSubLabels")) {
      subLabels = opt.getAttribute("buttonSubLabels").split(/\s*,\s*/);
    }
    
    newOrder.push({
      id: opt.getAttribute("buttonId"),
      label: opt.getAttribute("buttonLabel"),
      collapsed: (opt.getAttribute("buttonCollapsed") == "true"),
      iconMode: parseInt(opt.getAttribute("buttonIconMode")),
      subLabels: subLabels,
      renamed: (opt.getAttribute("buttonRenamed") == "true")
    });
  }

  let obj = {};
  obj["placement" + prefSuffix] = newOrder;
  await browser.storage.local.set(obj);
}

function hideSelectedButtons(list) {
  if (list.selectedIndex == -1) return false;

  let hiddenMsg = browser.i18n.getMessage("mhcHidden");
  let opts = list.selectedOptions;

  let hide = !(opts[0].getAttribute("buttonCollapsed") == "true"); //adopt top button for hidden mode
  for (let i = opts.length - 1; i >= 0; i--) {
    let opt = opts[i];
    if (hide) {
      if (opt.id.indexOf("mhcSPC") == 0 || opt.id.indexOf("mhcSEP") == 0) {
        list.removeChild(opt);
      } else {
        opt.textContent = opt.getAttribute("buttonLabel") + " " + hiddenMsg;
        opt.setAttribute("buttonCollapsed", true);
        opt.classList.add("option-hidden");
        setButtonTextAndStyle();
        updateIconModeButtons();
      }
    } else {
      opt.textContent = opt.getAttribute("buttonLabel");
      opt.setAttribute("buttonCollapsed", false);
      opt.classList.remove("option-hidden");
      setButtonTextAndStyle();
      updateIconModeButtons();
    }
  }

  return true;
}

function toggleIconForSelectedButtons(list) {
  if (list.selectedIndex == -1) return false;
  let opts = list.selectedOptions;
  let collapsed = opts[0].getAttribute("buttonCollapsed") == "true";
  let mode = collapsed ? -1 : parseInt(opts[0].getAttribute("buttonIconMode"));

  switch (mode) {
    case -1: //hidden -> icon
      hideSelectedButtons(list);
      setIconModeForSelectedButtons(list, 1);
      break;
    case 0: //icon and text -> text
      setIconModeForSelectedButtons(list, 2);
      break;
    case 1: //icon only -> hidden
      hideSelectedButtons(list);
      break;
    case 2: //text only -> icon and text
      setIconModeForSelectedButtons(list, 0);
      break;
    default:
      break;
  }

  return true;
}

function toggleTextForSelectedButtons(list) {
  if (list.selectedIndex == -1) return false;
  let opts = list.selectedOptions;
  let collapsed = opts[0].getAttribute("buttonCollapsed") == "true";
  let mode = collapsed ? -1 : parseInt(opts[0].getAttribute("buttonIconMode"));

  switch (mode) {
    case -1: //hidde -> text
      hideSelectedButtons(list);
      setIconModeForSelectedButtons(list, 2);
      break;
    case 0: //icon and text -> icon
      setIconModeForSelectedButtons(list, 1);
      break;
    case 1: //icon only -> icon and text
      setIconModeForSelectedButtons(list, 0);
      break;
    case 2: //text only -> hidden
      hideSelectedButtons(list);
      break;
    default:
      break;
  }

  return true;
}

function renameButtons(list) {
  if (list.selectedOptions.length != 1) return false;
  let opt = list.selectedOptions[0];
  let label = opt.getAttribute("buttonSubLabels") ? opt.getAttribute("buttonSubLabels") : opt.getAttribute("buttonLabel");
  let numLabels = label.split(/\s*,\s*/).length;
  let msg = "Input new label.";
  let newLabel = window.prompt(msg, label);
  let renamed = false;
  if (newLabel && newLabel.split(/\s*,\s*/).length == numLabels) {
    let newLabelArray = newLabel.split(/\s*,\s*/);
    opt.setAttribute("buttonSubLabels", newLabel);
    opt.setAttribute("buttonLabel", newLabelArray[0]);
    setButtonTextAndStyle();
    opt.setAttribute("buttonRenamed", true);
    renamed = true;
  }
  
  return renamed;
}

function setIconModeForSelectedButtons(list, mode) {
  if (list.selectedIndex == -1) return false;
  let opts = list.selectedOptions;
  for (let i = 0; i < opts.length; i++) {
    let opt = opts[i];
    opt.setAttribute("buttonIconMode", mode);
  }

  setButtonTextAndStyle();
  updateIconModeButtons();
  return true;
}

async function resetButtonOrder() {
  let obj = {};
  obj["placement" + prefSuffix] = [];
  await browser.storage.local.set(obj);

  obj["icon_mode" + prefSuffix] = 1;
  await browser.storage.local.set(obj);

  let msg = browser.i18n.getMessage("mhcRestartMsg");
  alert(msg);
}

function addSpacer(list, type) {
  if (list.selectedIndex == -1) return false;
  let refOpt = list.selectedOptions[0];
  let label = "";
  if (type == "mhcSPC") {
    label = browser.i18n.getMessage("mhcSpace");
  } else if (type == "mhcSEP") {
    label = browser.i18n.getMessage("mhcSeparator");
  }
  let opt = document.createElement("option");
  opt.textContent = label;
  opt.setAttribute("buttonCollapsed", false);
  let date = new Date();
  let id = date.getTime();
  opt.id = type + "-" + id;
  opt.setAttribute("buttonLabel", label);
  opt.setAttribute("buttonId", opt.id);
  opt.setAttribute("buttonIconMode", 0);

  list.insertBefore(opt, refOpt);

  return true;
}

function updateShowHideButton() {
  let box = document.getElementById("placement");
  let opt = box.selectedOptions[0];
  let localeKey = "";
  if (opt) {
    let spacerSelected = opt.id.indexOf("mhcSPC") == 0 || opt.id.indexOf("mhcSEP") == 0;
    if (spacerSelected) {
      localeKey = "mhcDel";
    } else {
      let hide = !(opt.getAttribute("buttonCollapsed") == "true");
      localeKey = hide ? "mhcHide" : "mhcShow";
    }
  } else { //hiding space and separator case. opt is removed.
    localeKey = "mhcHide";
  }

  document.getElementById("buttonHide").value = browser.i18n.getMessage(localeKey);
}

function updateIconModeButtons() {
  let box = document.getElementById("placement");
  let opt = box.selectedOptions[0];
  if (!opt) return false;
  
  let mode = 0;
  let modeButtonDisabled = opt.id.indexOf("mhcSPC") == 0 || opt.id.indexOf("mhcSEP") == 0;
  if (modeButtonDisabled || opt.getAttribute("buttonCollapsed") == "true") {
    mode = -1;
  } else {
    mode = parseInt(opt.getAttribute("buttonIconMode"));
  }
  
  document.getElementById("buttonIcon").disabled = modeButtonDisabled;
  document.getElementById("buttonText").disabled = modeButtonDisabled;
  
  switch (mode) {
    case -1:
      document.getElementById("buttonIcon").setAttribute("class", "button-fix-side");
      document.getElementById("buttonText").setAttribute("class", "button-fix-side");
      break;
    case 0:
      document.getElementById("buttonIcon").setAttribute("class", "button-fix-side-toggled");
      document.getElementById("buttonText").setAttribute("class", "button-fix-side-toggled");
      break;
    case 1:
      document.getElementById("buttonIcon").setAttribute("class", "button-fix-side-toggled");
      document.getElementById("buttonText").setAttribute("class", "button-fix-side");
      break;
    case 2:
      document.getElementById("buttonIcon").setAttribute("class", "button-fix-side");
      document.getElementById("buttonText").setAttribute("class", "button-fix-side-toggled");
      break;
    default:
      break;
  }
}

function updateIconModeButtonsVisibility() {
  let mode = document.mhc_form.icon_mode.value;
  let box = document.getElementById("placement");
  let opt = box.selectedOptions[0];
  let spacerSelected = opt ? opt.id.indexOf("mhcSPC") == 0 || opt.id.indexOf("mhcSEP") == 0 : false;
  if (mode != "-1" || spacerSelected) {
    document.getElementById("buttonHideArea").classList.remove("box-collapse");
    document.getElementById("buttonIconModeArea").classList.add("box-collapse");
  } else {
    document.getElementById("buttonHideArea").classList.add("box-collapse");
    document.getElementById("buttonIconModeArea").classList.remove("box-collapse");
  }
}

init();
